/* Definitions for rtnppd */
/* $Id: rtnppd.h,v 1.12 1999/08/21 08:14:01 vasim Exp $ */

#define MAX_PACKET_SIZE 4160
#define DEFAULT_PORT 4782
#define DEFAULT_MAIN_CONFIG "/usr/local/etc/rtnppd/rtnppd.conf"
#define DEFAULT_DEV_CONFIG "/usr/local/etc/rtnppd/tnpp_dev.conf"
#define DEFAULT_TRANS_CONFIG "/usr/local/etc/rtnppd/tnpp_trans.conf"
#define DEFAULT_PID_FILE "/var/run/rtnppd.pid"
#define DEFAULT_SOCKET_IN_FILE "/var/run/rtnppd.queue.in"
#define DEFAULT_SOCKET_OUT_FILE "/var/run/rtnppd.queue.out"
#define DEFAULT_QUEUE_FILE "/var/spool/rtnppd/queue"
#define SERIAL_TABLE_NUM 64
#define CAP_TRANS_NUM 4

/* TNPP-packets */
typedef struct tnpp_packet {
  int status;   /* Status of this packet
		 * 0 - waiting for transmit
		 * 1 - transmitting
		 * 2 - waiting for ack
		 * 3 - RS wait (thold seconds) */
  unsigned char bytes[MAX_PACKET_SIZE]; /* Packet data */
  int thold;    /* Hold timer for RS */
  int chold;    /* Hold counter */
  int offset;   /* Current byte in packet for transmission */
  int length;   /* Number of bytes in packet */
  int encrypt;  /* Encrypted packet */
  int type;     /* Type of packet */
  struct timeval begintime;
  struct tnpp_packet *prev; /* Previous packet in queue */
  struct tnpp_packet *next; /* Next packet in queue */
} tnpp_packet;

/* blowfish's data */
typedef struct tnpp_bf {
  bf_subkeys skeys;
} tnpp_bf;

/* TNPP-devices (serial ports & sockets) */
typedef struct tnpp_device {
  int fd;       /* File descriptor for this device */
  int type;     /* Type of device
		 * 0 - Serial port
		 * 1 - socket */
  char *name;   /* Name of device for logging&etc */
  int addr;     /* TNPP-address of this device */
  char *device; /* Name of file for serial device */
  int serial_clocal;  /* CLOCAL mode (no carrier detect) */
  int serial_crtscts; /* RTS/CTS flow control */
  int serial_dtrtoggle;    /* must DTR toggle at connect */
  int serial_speed;   /* Baud for serial device */
  int serials[SERIAL_TABLE_NUM]; /* serial tables for incoming packets */
  int serials_end;    /* Pointer in serials[] */
  int last_serial;    /* last serial number for this link */
  int transparentcrc; /* Transparent checksum */
  struct sockaddr_in peer; /* Peer address for socket-device */
  tnpp_packet *head;  /* First packet in queue */
  tnpp_packet *tail;  /* Last packet in queue */
  int num_packs;      /* Number of packets in queue */
  int status;   /* Status of receiver
		 * 0 - idle
		 * 1 - receiving packet
		 * 2 - receiving first byte of checksum
		 * 3 - receiving last byte of checksum
		 * 4 - waiting EOT
		 * 5 - waiting EOT after tnpp_init (must send empty packet)
		 * 6 - receiving second byte of checksum (for transparency)
		 * 7 - receiving third byte of checksum (for transparency)
		 * 255 - receiver blocked */
  int statusout; /* Status of transmitter
		  * 0 - idle
		  * 1 - sending packet
		  * 2 - send ENQ only, not packets
		  * 3 - send EOT, then status = 2
		  * 255 - transmitter blocked */
  unsigned char bytes[MAX_PACKET_SIZE]; /* Receive buffer */
  int offset;   /* Offset in receive buffer */
  int tidle;    /* Idle timer (before send ENQ) */
  int tict;     /* Intercharacting timer (input) */
  int tictout;  /* Intercharacting timer (output) */
  int tnri;     /* Timer for response wait */
  int cretry;   /* Retry counter (output) */
  int cenq;     /* Retry counter on ENQ (output) */
  int encrypt;  /* Encrypt packet for this device */
  int broken_fish;    /* Use my own "modified" blowfish algorithm */
  tnpp_bf *bf;  /* Blowfish's data (if encrypt != 0) */
  char *key_file;     /* Name of file with blowfish's key (up to 448 bits) */
  char *page;   /* Page this message, when link failures */
  int page_already;
  unsigned long int pktin;  /* Counter of inbound packets (for statistic) */
  unsigned long int pktout; /* Counter of outbound packets (for statistic) */
  unsigned long int pktinerr;
  unsigned long int pktouterr;
  struct tnpp_device *prev; /* Previous device */
  struct tnpp_device *next; /* Next device */
} tnpp_device;

/* Routing records */
typedef struct tnpp_route {
  int low_addr, high_addr; /* Destination addresses (from l_addr to h_addr) */
  tnpp_device *dev;   /* Router for this addresses */
  struct tnpp_route *prev;   /* Previous routing record */
  struct tnpp_route *next;   /* Last route record */
} tnpp_route;

/* CAP function translation records */
typedef struct cap_translate {
  unsigned long low;         /* Lowest CAP code in range */
  unsigned long high;        /* Highest CAP code in range */
  int change;                /* change to CAP function (-1 or +1) */
} cap_translate;

/* Translation records */
typedef struct tnpp_translate {
  int old_addr;       /* Change destination address in packets for old_addr */
  int new_addr;       /* to new_addr */
  int logging;        /* Full packet log need after translate */
  struct tnpp_translate *prev;
  struct tnpp_translate *next;
  cap_translate cap[CAP_TRANS_NUM];
} tnpp_translate;

#define SOH '\01'
#define STX '\02'
#define ETX '\03'
#define EOT '\04'
#define ENQ '\05'
#define ACK '\06'
#define NAK '\025'
#define CAN '\030'
#define RS  '\036'
#define SUB '\032'
